/*
 * Copyright (c) 2003-2004 Linuxant inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

#define OS_NOSTRINGREDEFS

#include "oscompat.h"
#include "osservices.h"
#include "ndiswrap.h"
#include "netndis.h"
#include "osusb.h"

#include <linux/pci.h>
#include <linux/usb.h>
#include <linux/kmod.h>
#include <linux/ctype.h>

MODULE_AUTHOR("Copyright (C) 2003-2004 Linuxant inc.");
MODULE_DESCRIPTION(DLDRDESC);
MODULE_LICENSE("see LICENSE file; Copyright (c)2003-2004 Linuxant inc.");
MODULE_INFO(supported, "yes");

DECLARE_MUTEX(dldr_init_mutex);
EXPORT_SYMBOL_NOVERS(dldr_init_mutex);

static int __init dldr_init (void)
{
	int err;
    char *argv[4], *envp[3];

	OsTestPrintf("%s\n", __FUNCTION__);

	netndis_init();

    envp [0] = "HOME=/";
    envp [1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"DLDRSBINDIR;
    envp [2] = 0;
    argv[0] = DLDRSBINDIR"/dldrconfig";
    argv[1] = "--auto";
    argv[2] = "--mod-pre-init";
    argv[3] = 0;

	err = OsForkWait(argv[0], argv, envp);
    if(err) {
        printk(KERN_ERR"%s: OsForkWait %s returned %d\n", __FUNCTION__, argv[0], err);
    }

	err = NdisWrapInit();
	if(err) {
		netndis_exit();
		return err;
	}
	
	return 0;
}


static void __exit dldr_cleanup (void)
{
	OsTestPrintf("%s\n", __FUNCTION__);

	NdisWrapExit();
	netndis_exit();
}

module_init(dldr_init);
module_exit(dldr_cleanup);

const static netndis_usb_driver *usb_driver;

void netndis_usb_register_driver(const netndis_usb_driver *drv)
{
	usb_driver = drv;
}

void netndis_usb_unregister_driver(void)
{
	usb_driver = NULL;
}

__shimcall__ INT OsUsbSetConfiguration(PUSBOSHAL pUsbOsHal, INT Configuration)
{
	return usb_driver->set_configuration(pUsbOsHal, Configuration);
}

__shimcall__ INT OsUsbMakeBulkOrIntTransfer(PUSBOSPIPE pUsbOsPipe, PVOID userContext, PVOID TransferBuffer, ULONG TransferBufferLength)
{
	return usb_driver->make_bulk_or_int_transfer(pUsbOsPipe, userContext, TransferBuffer, TransferBufferLength);
}

__shimcall__ INT OsUsbMakeControlTransfer(PUSBOSHAL pUsbOsHal, BOOLEAN directionIn, OSUSB_TYPE usbType, OSUSB_RECIP usbRecip, PVOID userContext, UINT8 Request, UINT16 Value, UINT16 Index, PVOID TransferBuffer, ULONG TransferBufferLength)
{
	return usb_driver->make_control_transfer(pUsbOsHal, directionIn, usbType, usbRecip, userContext, Request, Value, Index, TransferBuffer, TransferBufferLength);
}

__shimcall__ PUSBOSPIPE OsUsbCreatePipe(PUSBOSHAL pUsbOsHal, UCHAR bEndpointAddress, UCHAR bmAttributes, UINT16 wMaxPacketSize, UCHAR bInterval)
{
	return usb_driver->create_pipe(pUsbOsHal, bEndpointAddress, bmAttributes, wMaxPacketSize, bInterval);
}

__shimcall__ VOID OsUsbClaimInterface(PUSBOSHAL pUsbOsHal, UCHAR InterfaceNumber, UCHAR AlternateSetting)
{
	usb_driver->claim_interface(pUsbOsHal, InterfaceNumber, AlternateSetting);
}

__shimcall__ INT OsUsbResetPort(PUSBOSHAL pUsbOsHal)
{
	return usb_driver->reset_port(pUsbOsHal);
}

__shimcall__ BOOLEAN OsUsbIsHighSpeed(PUSBOSHAL pUsbOsHal)
{
	return usb_driver->is_high_speed(pUsbOsHal);
}

__shimcall__ INT OsUsbAbortPipe(PUSBOSPIPE pUsbOsPipe)
{
	return usb_driver->abort_pipe(pUsbOsPipe);
}

__shimcall__ void OsUsbHaltInstance(PUSBOSHAL pUsbOsHal)
{
	return usb_driver->halt_instance(pUsbOsHal);
}

__shimcall__ void OsUsbAbortUrbWithContext (PUSBOSHAL pUsbOsHal, PVOID userContext)
{
	usb_driver->abort_urb_with_context(pUsbOsHal, userContext);
}

__shimcall__ INT OsUsbClearHalt(PUSBOSPIPE pUsbOsPipe)
{
	return usb_driver->clear_halt(pUsbOsPipe);
}

__shimcall__ INT OsUsbGetDescriptor(PUSBOSHAL pUsbOsHal, UCHAR DescType, UCHAR DescIndex, PVOID TransferBuffer, ULONG TransferBufferLength)
{
	return usb_driver->get_descriptor(pUsbOsHal, DescType, DescIndex, TransferBuffer, TransferBufferLength);
}


EXPORT_SYMBOL_NOVERS(NdisWrapHaltAdapter);
EXPORT_SYMBOL_NOVERS(NdisWrapGetVendorDesc);
EXPORT_SYMBOL_NOVERS(NdisWrapEnter);
EXPORT_SYMBOL_NOVERS(NdisWrapLeave);
EXPORT_SYMBOL_NOVERS(CallbackOsUsbBulkOrIntTransferCompleted);
EXPORT_SYMBOL_NOVERS(CallbackOsUsbControlTransferCompleted);
EXPORT_SYMBOL_NOVERS(NdisWrapCreateAdapter);
EXPORT_SYMBOL_NOVERS(NdisWrapNotifyBusDeregister);
EXPORT_SYMBOL_NOVERS(NdisWrapDestroyAdapter);
EXPORT_SYMBOL_NOVERS(NdisWrapNotifyAdapterRemoved);
EXPORT_SYMBOL_NOVERS(NdisWrapGetDMAMask);
EXPORT_SYMBOL_NOVERS(NdisWrapSetPowerState);
EXPORT_SYMBOL_NOVERS(NdisWrapStatusToErrno);
EXPORT_SYMBOL_NOVERS(netndis_usb_register_driver);
EXPORT_SYMBOL_NOVERS(netndis_usb_unregister_driver);

